@@ -6,19 +6,19 @@
 //!
 //! This method require floating point power function.
 use crate::utility::prelude::*;
-use alloc::{vec, vec::Vec};
+use alloc::vec::Vec;
 use core::marker::PhantomData;
 
 /// Real-coded Genetic Algorithm settings.
-pub struct Rga<R: Respond> {
+pub struct Rga<F: Fitness> {
     cross: f64,
     mutate: f64,
     win: f64,
     delta: f64,
-    _marker: PhantomData<R>,
+    _marker: PhantomData<F>,
 }
 
-impl<R: Respond> Rga<R> {
+impl<F: Fitness> Rga<F> {
     impl_builders! {
         /// Crossing probability.
         fn cross(f64)
@@ -31,7 +31,7 @@ impl<R: Respond> Rga<R> {
     }
 }
 
-impl<R: Respond> Default for Rga<R> {
+impl<F: Fitness> Default for Rga<F> {
     fn default() -> Self {
         Self {
             cross: 0.95,
@@ -43,8 +43,8 @@ impl<R: Respond> Default for Rga<R> {
     }
 }
 
-impl<R: Respond> Setting for Rga<R> {
-    type Algorithm = Method<R>;
+impl<F: Fitness> Setting for Rga<F> {
+    type Algorithm = Method<F>;
 
     fn algorithm(self) -> Self::Algorithm {
         Method {
@@ -57,30 +57,73 @@ impl<R: Respond> Setting for Rga<R> {
         }
     }
 
-    fn default_basic() -> BasicSetting {
-        BasicSetting {
-            pop_num: 500,
-            ..Default::default()
-        }
+    fn default_pop() -> usize {
+        500
     }
 }
 
 /// Real-coded Genetic Algorithm type.
-pub struct Method<R: Respond> {
+pub struct Method<F: Fitness> {
     cross: f64,
     mutate: f64,
     win: f64,
     delta: f64,
-    fitness_new: Vec<R>,
+    fitness_new: Vec<F>,
     pool_new: Array2<f64>,
 }
 
-impl<R: Respond> Method<R> {
-    fn crossover<F>(&mut self, ctx: &mut Context<F>)
+impl<Ft: Fitness> Method<Ft> {
+    fn get_delta<F>(&self, ctx: &Context<F>, y: f64) -> f64
     where
-        F: ObjFunc<Respond = R>,
+        F: ObjFunc<Fitness = Ft>,
     {
-        for i in (0..(ctx.pop_num() - 1)).step_by(2) {
+        let r = if ctx.gen < 100 {
+            ctx.gen as f64 / 100.
+        } else {
+            1.
+        };
+        #[cfg(all(feature = "std", not(feature = "libm")))]
+        let pow_f = (1. - r).powf(self.delta);
+        #[cfg(feature = "libm")]
+        let pow_f = libm::pow(1. - r, self.delta);
+        y * ctx.rng.rand() * pow_f
+    }
+}
+
+impl<F: ObjFunc> Algorithm<F> for Method<F::Fitness> {
+    #[inline(always)]
+    fn init(&mut self, ctx: &mut Context<F>) {
+        self.pool_new = Array2::zeros(ctx.pool.raw_dim());
+        self.fitness_new = ctx.fitness.clone();
+    }
+
+    #[inline(always)]
+    fn generation(&mut self, ctx: &mut Context<F>) {
+        // Select
+        for i in 0..ctx.pop_num() {
+            let (j, k) = {
+                let mut v = [i, 0, 0];
+                ctx.rng.vector(&mut v, 1, 0..ctx.pop_num());
+                (v[1], v[2])
+            };
+            if ctx.fitness[j] > ctx.fitness[k] && ctx.rng.maybe(self.win) {
+                self.fitness_new[i] = ctx.fitness[k].clone();
+                self.pool_new
+                    .slice_mut(s![i, ..])
+                    .assign(&ctx.pool.slice(s![k, ..]));
+            } else {
+                self.fitness_new[i] = ctx.fitness[j].clone();
+                self.pool_new
+                    .slice_mut(s![i, ..])
+                    .assign(&ctx.pool.slice(s![j, ..]));
+            }
+            ctx.fitness = self.fitness_new.clone();
+            ctx.pool.assign(&self.pool_new);
+            let i = ctx.rng.int(0..ctx.pop_num());
+            ctx.assign_from_best(i);
+        }
+        // Crossover
+        for i in (0..ctx.pop_num() - 1).step_by(2) {
             if !ctx.rng.maybe(self.cross) {
                 continue;
             }
@@ -109,35 +152,15 @@ impl<R: Respond> Method<R> {
                             var
                         };
                     }
-                    let f = ctx.func.fitness(v.as_slice().unwrap(), &ctx.report);
+                    let f = ctx.func.fitness(v.as_slice().unwrap(), ctx.adaptive);
                     (f, v)
                 })
                 .collect::<Vec<_>>();
-            v.sort_unstable_by(|(a, _), (b, _)| a.value().partial_cmp(&b.value()).unwrap());
+            v.sort_unstable_by(|(a, _), (b, _)| a.partial_cmp(b).unwrap());
             ctx.assign_from(i, v[0].0.clone(), &v[0].1);
             ctx.assign_from(i + 1, v[1].0.clone(), &v[1].1);
         }
-    }
-
-    fn get_delta<F>(&self, ctx: &Context<F>, y: f64) -> f64
-    where
-        F: ObjFunc<Respond = R>,
-    {
-        let r = match ctx.task {
-            Task::MaxGen(v) if v > 0 => ctx.report.gen as f64 / v as f64,
-            _ => 1.,
-        };
-        #[cfg(all(feature = "std", not(feature = "libm")))]
-        let pow_f = (1. - r).powf(self.delta);
-        #[cfg(feature = "libm")]
-        let pow_f = libm::pow(1. - r, self.delta);
-        y * ctx.rng.rand() * pow_f
-    }
-
-    fn mutate<F>(&mut self, ctx: &mut Context<F>)
-    where
-        F: ObjFunc<Respond = R>,
-    {
+        // Mutate
         for i in 0..ctx.pop_num() {
             if !ctx.rng.maybe(self.mutate) {
                 continue;
@@ -152,51 +175,4 @@ impl<R: Respond> Method<R> {
         }
         ctx.find_best();
     }
-
-    fn select<F>(&mut self, ctx: &mut Context<F>)
-    where
-        F: ObjFunc<Respond = R>,
-    {
-        for i in 0..ctx.pop_num() {
-            let (j, k) = {
-                let mut v = [i, 0, 0];
-                ctx.rng.vector(&mut v, 1, 0..ctx.pop_num());
-                (v[1], v[2])
-            };
-            if ctx.fitness[j].value() > ctx.fitness[k].value() && ctx.rng.maybe(self.win) {
-                self.fitness_new[i] = ctx.fitness[k].clone();
-                self.pool_new
-                    .slice_mut(s![i, ..])
-                    .assign(&ctx.pool.slice(s![k, ..]));
-            } else {
-                self.fitness_new[i] = ctx.fitness[j].clone();
-                self.pool_new
-                    .slice_mut(s![i, ..])
-                    .assign(&ctx.pool.slice(s![j, ..]));
-            }
-            ctx.fitness = self.fitness_new.clone();
-            ctx.pool.assign(&self.pool_new);
-            let i = ctx.rng.int(0..ctx.pop_num());
-            ctx.assign_from_best(i);
-        }
-    }
-}
-
-impl<F, R> Algorithm<F> for Method<R>
-where
-    F: ObjFunc<Respond = R>,
-    R: Respond,
-{
-    #[inline(always)]
-    fn init(&mut self, ctx: &mut Context<F>) {
-        self.pool_new = Array2::zeros(ctx.pool.raw_dim());
-        self.fitness_new = vec![R::INFINITY; ctx.fitness.len()];
-    }
-
-    #[inline(always)]
-    fn generation(&mut self, ctx: &mut Context<F>) {
-        self.select(ctx);
-        self.crossover(ctx);
-        self.mutate(ctx);
-    }
 }
