@@ -1,9 +1,9 @@
-use crate::{utility::Respond, Report};
+use crate::utility::prelude::*;
 
-/// The base of the objective function.
+/// A trait for the objective function.
 ///
 /// ```
-/// use metaheuristics_nature::{ObjFunc, Report};
+/// use metaheuristics_nature::ObjFunc;
 ///
 /// struct MyFunc([f64; 3], [f64; 3]);
 ///
@@ -15,42 +15,48 @@ use crate::{utility::Respond, Report};
 ///
 /// impl ObjFunc for MyFunc {
 ///     type Result = f64;
-///     type Respond = f64;
+///     type Fitness = f64;
 ///
-///     fn fitness(&self, v: &[f64], _: &Report) -> Self::Respond {
+///     fn fitness(&self, v: &[f64], _: f64) -> Self::Fitness {
 ///         v[0] * v[0] + v[1] * v[1] + v[2] * v[2]
 ///     }
 ///
 ///     fn result(&self, v: &[f64]) -> Self::Result {
-///         self.fitness(v, &Default::default())
+///         self.fitness(v, 0.)
 ///     }
 ///
 ///     fn ub(&self) -> &[f64] {
 ///         &self.1
 ///     }
+///
 ///     fn lb(&self) -> &[f64] {
 ///         &self.0
 ///     }
 /// }
 /// ```
-/// The objective function returns fitness value that used to evaluate the objective.
 ///
+/// The objective function returns fitness value that used to evaluate the objective.
 /// The lower bound and upper bound represents the number of variables at the same time.
 ///
-/// This trait is designed as immutable.
+/// This trait is designed as immutable and there should only has shared data.
 pub trait ObjFunc: Sync + Send + 'static {
     /// The result type.
     type Result;
     /// Representation of the fitness value.
-    type Respond: Respond;
+    type Fitness: Fitness;
 
     /// Return fitness, the smaller value represents a good result.
     ///
     /// # How to design the fitness value?
     ///
     /// Regularly, the evaluation value **should not** lower than zero,
-    /// because a negative infinity can directly break the result.
-    /// Instead, a positive enhanced floating point value is the better choice.
+    /// because it is not easy to control with multiplication,
+    /// and a negative infinity can directly break the result.
+    /// Instead, a positive enhanced floating point value is the better choice,
+    /// and the zero is the best result.
+    ///
+    /// In another hand, some marker can help you to compare with other design,
+    /// please see [`Fitness`] for more information.
     ///
     /// # Penalty
     ///
@@ -71,7 +77,12 @@ pub trait ObjFunc: Sync + Send + 'static {
     /// So that, we use secondary evaluation function to measure the result from other requirements,
     /// we call it "constraint" or "penalty function".
     /// The penalty value usually multiply a weight factor for increasing its influence.
-    fn fitness(&self, v: &[f64], r: &Report) -> Self::Respond;
+    ///
+    /// # Adaptive Value
+    ///
+    /// Sometimes a value that adjust with converge states can help to restrict the searching.
+    /// The "adaptive function" can be set in [`SolverBuilder::adaptive`] method.
+    fn fitness(&self, v: &[f64], f: f64) -> Self::Fitness;
 
     /// Return the final result of the problem.
     ///
