@@ -1,34 +1,33 @@
 use crate::utility::prelude::*;
-use alloc::{sync::Arc, vec, vec::Vec};
+use alloc::{vec, vec::Vec};
 
 /// The base class of algorithms.
 ///
 /// This type provides a shared dataset if you want to implement a new method.
 ///
 /// Please see [`Algorithm`] for the implementation.
+#[non_exhaustive]
 pub struct Context<F: ObjFunc> {
     /// Random number generator.
     pub rng: Rng,
-    /// Termination condition.
-    pub task: Task,
     /// The best variables.
     pub best: Array1<f64>,
-    /// Current fitness of all individuals.
-    pub fitness: Vec<F::Respond>,
+    /// Best fitness.
+    pub best_f: F::Fitness,
     /// Current variables of all individuals.
     pub pool: Array2<f64>,
-    /// The current information of the algorithm.
-    pub report: Report,
-    pub(crate) reports: Vec<Report>,
-    pub(crate) rpt: u64,
-    pub(crate) average: bool,
-    pub(crate) adaptive: Adaptive,
+    /// Current fitness of all individuals.
+    pub fitness: Vec<F::Fitness>,
+    /// Adaptive factor.
+    pub adaptive: f64,
+    /// Generation.
+    pub gen: u64,
     /// The objective function.
-    pub func: Arc<F>,
+    pub func: F,
 }
 
 impl<F: ObjFunc> Context<F> {
-    pub(crate) fn new(func: F, s: BasicSetting) -> Self {
+    pub(crate) fn new(func: F, seed: Option<u128>, pop_num: usize) -> Self {
         let dim = func.lb().len();
         assert_eq!(
             dim,
@@ -36,17 +35,14 @@ impl<F: ObjFunc> Context<F> {
             "different dimension of the variables!"
         );
         Self {
-            rng: Rng::new(s.seed),
-            task: s.task,
+            rng: Rng::new(seed),
             best: Array1::zeros(dim),
-            fitness: vec![F::Respond::INFINITY; s.pop_num],
-            pool: Array2::zeros((s.pop_num, dim)),
-            report: Report::default(),
-            reports: Vec::new(),
-            rpt: s.rpt,
-            average: s.average,
-            adaptive: s.adaptive,
-            func: Arc::new(func),
+            best_f: F::Fitness::default(),
+            pool: Array2::zeros((pop_num, dim)),
+            fitness: vec![F::Fitness::default(); pop_num],
+            adaptive: 0.,
+            gen: 0,
+            func,
         }
     }
 
@@ -80,9 +76,10 @@ impl<F: ObjFunc> Context<F> {
 
     /// Get fitness from individual `i`.
     pub fn fitness(&mut self, i: usize) {
-        self.fitness[i] = self
-            .func
-            .fitness(self.pool.slice(s![i, ..]).as_slice().unwrap(), &self.report);
+        self.fitness[i] = self.func.fitness(
+            self.pool.slice(s![i, ..]).as_slice().unwrap(),
+            self.adaptive,
+        );
     }
 
     pub(crate) fn init_pop(&mut self) {
@@ -92,56 +89,62 @@ impl<F: ObjFunc> Context<F> {
         let mut fitness = self.fitness.clone();
         let zip = Zip::from(&mut fitness).and(pool.axis_iter(Axis(0)));
         #[cfg(not(feature = "parallel"))]
-        {
-            zip.for_each(|f, v| {
-                *f = self.func.fitness(v.to_slice().unwrap(), &self.report);
-            });
-            self.find_best();
-        }
+        let _ = zip.for_each(|f, v| {
+            *f = self.func.fitness(v.to_slice().unwrap(), self.adaptive);
+        });
         #[cfg(feature = "parallel")]
         {
             let (f, v) = zip
                 .into_par_iter()
                 .map(|(f, v)| {
-                    *f = self.func.fitness(v.to_slice().unwrap(), &self.report);
-                    (f.value(), v)
+                    *f = self.func.fitness(v.to_slice().unwrap(), self.adaptive);
+                    (f, v)
                 })
                 .min_by(|(a, _), (b, _)| a.partial_cmp(b).unwrap())
                 .unwrap();
-            self.set_best_from(f, v);
+            self.set_best_from(f.clone(), v);
         }
         self.pool = pool;
         self.fitness = fitness;
+        #[cfg(not(feature = "parallel"))]
+        {
+            let mut best = 0;
+            for i in 1..self.pop_num() {
+                if self.fitness[i] < self.fitness[best] {
+                    best = i;
+                }
+            }
+            self.set_best(best);
+        }
     }
 
     /// Set the index to best.
     #[inline(always)]
     pub fn set_best(&mut self, i: usize) {
-        self.report.best_f = self.fitness[i].value();
-        self.report.best_feasible = self.fitness[i].feasible();
+        self.best_f = self.fitness[i].clone();
         self.best.assign(&self.pool.slice(s![i, ..]));
     }
 
     /// Set the fitness and variables to best.
     #[inline(always)]
-    pub fn set_best_from<'a, A>(&mut self, f: f64, v: A)
+    pub fn set_best_from<'a, A>(&mut self, f: F::Fitness, v: A)
     where
         A: AsArray<'a, f64>,
     {
-        self.report.best_f = f;
+        self.best_f = f;
         self.best.assign(&v.into());
     }
 
     /// Assign the index from best.
     #[inline(always)]
     pub fn assign_from_best(&mut self, i: usize) {
-        self.fitness[i] = F::Respond::from_value(self.report.best_f, self.report.best_feasible);
+        self.fitness[i] = self.best_f.clone();
         self.pool.slice_mut(s![i, ..]).assign(&self.best);
     }
 
     /// Assign the index from source.
     #[inline(always)]
-    pub fn assign_from<'a, A>(&mut self, i: usize, f: F::Respond, v: A)
+    pub fn assign_from<'a, A>(&mut self, i: usize, f: F::Fitness, v: A)
     where
         A: AsArray<'a, f64>,
     {
@@ -153,11 +156,11 @@ impl<F: ObjFunc> Context<F> {
     pub fn find_best(&mut self) {
         let mut best = 0;
         for i in 1..self.pop_num() {
-            if self.fitness[i].value() < self.fitness[best].value() {
+            if self.fitness[i] < self.fitness[best] {
                 best = i;
             }
         }
-        if self.fitness[best].value() < self.report.best_f {
+        if self.fitness[best] < self.best_f {
             self.set_best(best);
         }
     }
@@ -173,9 +176,4 @@ impl<F: ObjFunc> Context<F> {
             v
         }
     }
-
-    /// Record the performance.
-    pub(crate) fn report(&mut self) {
-        self.reports.push(self.report.clone());
-    }
 }
